"use strict";
// Tell React that this is not a server component, since it's likely to be used
// as the first client-side-only component in a tree
'use client';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BootstrapRoot = void 0;
const react_1 = __importStar(require("react"));
const recoil_1 = require("recoil");
const util_1 = require("./util");
/**
 * This component initializes the supplied root atom and its associated
 * bootstrapped atoms with the supplied bootstrap data.
 *
 * @param props.bootstrapData The bootstrap data to initialize bootstrapped
 *  atoms with.
 * @param props.rootAtom The root atom to store the bootstrap data, which in
 *  turn initializes all bootstrapped atoms associated with it.
 */
function BootstrapRoot({ children, bootstrapData, rootAtom, }) {
    // We get the loadable version of the root atom to check if it's in a loadable
    // state or not. Being in a loadable state tells us that the root atom has not
    // been initialized with bootstrap data yet, and that we should do so now.
    const [bootstrapDataLoadable] = (0, recoil_1.useRecoilStateLoadable)(rootAtom);
    // We get the list of known root atoms above this one. If any exist, then that
    // means this root is nested inside of another root. We need to pass along
    // the list of all roots to the nested context so that value hooks can check
    // if they are being called in the right component tree
    const parentBootstrapRoots = (0, react_1.useContext)(util_1.BootstrapRootsInScopeContext);
    // We use snapshots to set the root atom value instead of the typical
    // `useRecoilState` setter because the latter is asynchronous and would result
    // in bootstrap data not being available for the first render.
    const snapshot = (0, recoil_1.useRecoilSnapshot)();
    const gotoRecoilSnapshot = (0, recoil_1.useGotoRecoilSnapshot)();
    if (bootstrapDataLoadable.state === 'loading') {
        gotoRecoilSnapshot(snapshot.map(({ set }) => {
            set(rootAtom, bootstrapData);
        }));
    }
    // TODO: add some logic that puts the root atom back in a loadable state on
    // component unmount here.
    return (react_1.default.createElement(util_1.BootstrapRootsInScopeContext.Provider
    // We add this bootstrap root's root atom to the list of known roots.
    // Turns out, we can nest the same provider multiple times, and it scopes
    // itself so that we always get the right value in the right place.
    , { 
        // We add this bootstrap root's root atom to the list of known roots.
        // Turns out, we can nest the same provider multiple times, and it scopes
        // itself so that we always get the right value in the right place.
        value: [...parentBootstrapRoots, rootAtom] }, children));
}
exports.BootstrapRoot = BootstrapRoot;
//# sourceMappingURL=bootstrapRoot.js.map