"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bootstrappedAtom = void 0;
const recoil_1 = require("recoil");
const util_1 = require("./util");
// Bootstrapped atoms are initialized by creating a special hidden selector that
// we assign to the `default` prop in our atom we create. This selector is tied
// to the root atom, causing this atom's loadable state to match that of the
// root atom.
/**
 * Creates a bootstrapped atom for accessing bootstrap data.
 *
 * @param rootAtom The root atom containing the bootstrap data to initialize
 *  this atom with.
 * @param options Options here are the mostly the same as the options passed to
 *  the built-in `atom()` function in Recoil. The difference is that the
 *  `default` property is _not_ allowed, and there is a new `initialValue`
 *  function to replace `default`.
 * @param options.initialValue A function to initialize the bootstrapped atom
 *  with. This function is called at runtime with all of the bootstrap data
 *  passed to BoostrapRoot. The atom's value is then set to the value returned
 *  from this function.
 * @returns The bootstrapped atom that can then be passed to
 *  bootstrappedAtomValueHook to create a hook for safely accessing this data.
 *  The returned atom is a normal off-the-shelf Recoil atom, and can be used
 *  accordingly.
 */
function bootstrappedAtom(rootAtom, _a) {
    var { initialValue, 
    // We pull the key out so we can compute a derived key name for the selector
    key } = _a, options = __rest(_a, ["initialValue", "key"]);
    // Extra checking for vanilla JS users. This isn't possible in TypeScript
    if ('default' in options) {
        throw new Error('The "default" prop is not allowed in bootstrapped atoms. Use "initialValue" instead');
    }
    const newAtom = (0, recoil_1.atom)(Object.assign(Object.assign({}, options), { key, 
        // We set the default to a selector so that we can grab the bootstrap data
        // from its root atom, which is initialized in a <BootstrapRoot> component
        default: (0, recoil_1.selector)({
            key: `${key}:atomInitializer`,
            // TODO: do we need to guard against the first-run case when the rootAtom
            // is created but in a loadable state?
            get: ({ get }) => initialValue(get(rootAtom)),
        }) }));
    newAtom[util_1.rootAtomSymbol] = rootAtom;
    return newAtom;
}
exports.bootstrappedAtom = bootstrappedAtom;
//# sourceMappingURL=bootstrappedAtom.js.map