"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bootstrappedAtomValueHook = void 0;
const react_1 = require("react");
const recoil_1 = require("recoil");
const util_1 = require("./util");
/**
 * Creates a hook for accessing a bootstrapped atom's value safely.
 *
 * @param bootstrappedAtom - The bootstrapped atom to create the accessor hook for
 * @returns The hook that accesses the value.
 */
function bootstrappedAtomValueHook(bootstrappedAtom) {
    return () => {
        // Check if this bootstrapped atom's root atom is in scope. See comments for
        // the implementation of rootAtomSymbol and BootstrapRootsInScopeContext for
        // more details on how this works.
        const parentBootstrapRoots = (0, react_1.useContext)(util_1.BootstrapRootsInScopeContext);
        if (!parentBootstrapRoots.includes(bootstrappedAtom[util_1.rootAtomSymbol])) {
            throw new Error('Bootstrapped atom not loaded. Did you call this hook outside of a descendant of its <BootstrapRoot> component?');
        }
        return (0, recoil_1.useRecoilValue)(bootstrappedAtom);
    };
}
exports.bootstrappedAtomValueHook = bootstrappedAtomValueHook;
//# sourceMappingURL=bootstrappedAtomValueHook.js.map